@@ -21,12 +21,16 @@
 
 """Create and edit manifest for session contents"""
 
+from future_builtins import filter
+
 import re
 
+from duplicity import globals
 from duplicity import log
 from duplicity import globals
 from duplicity import util
 
+
 class ManifestError(Exception):
     """
     Exception raised when problem with manifest
@@ -38,7 +42,7 @@ class Manifest:
     """
     List of volumes and information about each one
     """
-    def __init__(self, fh = None):
+    def __init__(self, fh=None):
         """
         Create blank Manifest
 
@@ -50,8 +54,9 @@ class Manifest:
         """
         self.hostname = None
         self.local_dirname = None
-        self.volume_info_dict = {} # dictionary vol numbers -> vol infos
+        self.volume_info_dict = {}  # dictionary vol numbers -> vol infos
         self.fh = fh
+        self.files_changed = []
 
     def set_dirinfo(self):
         """
@@ -62,7 +67,7 @@ class Manifest:
         @return: manifest
         """
         self.hostname = globals.hostname
-        self.local_dirname = globals.local_path.name #@UndefinedVariable
+        self.local_dirname = globals.local_path.name  # @UndefinedVariable
         if self.fh:
             if self.hostname:
                 self.fh.write("Hostname %s\n" % self.hostname)
@@ -90,12 +95,13 @@ class Manifest:
             code = log.ErrorCode.hostname_mismatch
             code_extra = "%s %s" % (util.escape(globals.hostname), util.escape(self.hostname))
 
-        elif (self.local_dirname and self.local_dirname != globals.local_path.name): #@UndefinedVariable
+        elif (self.local_dirname and self.local_dirname != globals.local_path.name):  # @UndefinedVariable
             errmsg = _("Fatal Error: Backup source directory has changed.\n"
                        "Current directory: %s\n"
-                       "Previous directory: %s") % (self.local_dirname, globals.local_path.name) #@UndefinedVariable
+                       "Previous directory: %s") % (globals.local_path.name, self.local_dirname)  # @UndefinedVariable
             code = log.ErrorCode.source_dir_mismatch
-            code_extra = "%s %s" % (util.escape(self.local_dirname), util.escape(globals.local_path.name)) #@UndefinedVariable
+            code_extra = "%s %s" % (util.escape(globals.local_path.name),
+                                    util.escape(self.local_dirname))  # @UndefinedVariable
         else:
             return
 
@@ -107,6 +113,15 @@ class Manifest:
                          "--allow-source-mismatch switch to avoid seeing this "
                          "message"), code, code_extra)
 
+    def set_files_changed_info(self, files_changed):
+        if files_changed:
+            self.files_changed = files_changed
+
+        if self.fh:
+            self.fh.write("Filelist %d\n" % len(self.files_changed))
+            for fileinfo in self.files_changed:
+                self.fh.write("    %-7s  %s\n" % (fileinfo[1], Quote(fileinfo[0])))
+
     def add_volume_info(self, vi):
         """
         Add volume info vi to manifest and write to manifest
@@ -132,7 +147,7 @@ class Manifest:
         """
         try:
             del self.volume_info_dict[vol_num]
-        except:
+        except Exception:
             raise ManifestError("Volume %d not present in manifest" % (vol_num,))
 
     def to_string(self):
@@ -148,8 +163,13 @@ class Manifest:
         if self.local_dirname:
             result += "Localdir %s\n" % Quote(self.local_dirname)
 
+        result += "Filelist %d\n" % len(self.files_changed)
+        for fileinfo in self.files_changed:
+            result += "    %-7s  %s\n" % (fileinfo[1], Quote(fileinfo[0]))
+
         vol_num_list = self.volume_info_dict.keys()
         vol_num_list.sort()
+
         def vol_num_to_string(vol_num):
             return self.volume_info_dict[vol_num].to_string()
         result = "%s%s\n" % (result,
@@ -174,17 +194,48 @@ class Manifest:
         self.hostname = get_field("hostname")
         self.local_dirname = get_field("localdir")
 
-        next_vi_string_regexp = re.compile("(^|\\n)(volume\\s.*?)"
-                                           "(\\nvolume\\s|$)", re.I | re.S)
-        starting_s_index = 0
-        while 1:
-            match = next_vi_string_regexp.search(s[starting_s_index:])
-            if not match:
-                break
-            self.add_volume_info(VolumeInfo().from_string(match.group(2)))
-            starting_s_index += match.end(2)
+        highest_vol = 0
+        latest_vol = 0
+        vi_regexp = re.compile("(?:^|\\n)(volume\\s.*(?:\\n.*)*?)(?=\\nvolume\\s|$)", re.I)
+        vi_iterator = vi_regexp.finditer(s)
+        for match in vi_iterator:
+            vi = VolumeInfo().from_string(match.group(1))
+            self.add_volume_info(vi)
+            latest_vol = vi.volume_number
+            highest_vol = max(highest_vol, latest_vol)
+            log.Debug(_("Found manifest volume %s") % latest_vol)
+        # If we restarted after losing some remote volumes, the highest volume
+        # seen may be higher than the last volume recorded.  That is, the
+        # manifest could contain "vol1, vol2, vol3, vol2."  If so, we don't
+        # want to keep vol3's info.
+        for i in range(latest_vol + 1, highest_vol + 1):
+            self.del_volume_info(i)
+        log.Info(_("Found %s volumes in manifest") % latest_vol)
+
+        # Get file changed list - not needed if --file-changed not present
+        filecount = 0
+        if globals.file_changed is not None:
+            filelist_regexp = re.compile("(^|\\n)filelist\\s([0-9]+)\\n(.*?)(\\nvolume\\s|$)", re.I | re.S)
+            match = filelist_regexp.search(s)
+            if match:
+                filecount = int(match.group(2))
+            if filecount > 0:
+                def parse_fileinfo(line):
+                    fileinfo = line.strip().split()
+                    return (fileinfo[0], ''.join(fileinfo[1:]))
+
+                self.files_changed = list(map(parse_fileinfo, match.group(3).split('\n')))
+
+            if filecount != len(self.files_changed):
+                log.Error(_("Manifest file '%s' is corrupt: File count says %d, File list contains %d" %
+                            (self.fh.base if self.fh else "", filecount, len(self.files_changed))))
+                self.corrupt_filelist = True
+
         return self
 
+    def get_files_changed(self):
+        return self.files_changed
+
     def __eq__(self, other):
         """
         Two manifests are equal if they contain the same volume infos
@@ -204,7 +255,7 @@ class Manifest:
                 return False
 
         if (self.hostname != other.hostname or
-            self.local_dirname != other.local_dirname):
+                self.local_dirname != other.local_dirname):
             log.Notice(_("Manifests not equal because hosts or directories differ"))
             return False
 
@@ -221,7 +272,7 @@ class Manifest:
         Write string version of manifest to given path
         """
         assert not path.exists()
-        fout = path.open("w")
+        fout = path.open("wb")
         fout.write(self.to_string())
         assert not fout.close()
         path.setdata()
@@ -404,7 +455,7 @@ class VolumeInfo:
         """
         return not self.__eq__(other)
 
-    def contains(self, index_prefix, recursive = 1):
+    def contains(self, index_prefix, recursive=1):
         """
         Return true if volume might contain index
 
@@ -421,12 +472,14 @@ class VolumeInfo:
 
 
 nonnormal_char_re = re.compile("(\\s|[\\\\\"'])")
+
+
 def Quote(s):
     """
     Return quoted version of s safe to put in a manifest or volume info
     """
     if not nonnormal_char_re.search(s):
-        return s # no quoting necessary
+        return s  # no quoting necessary
     slist = []
     for char in s:
         if nonnormal_char_re.search(char):
@@ -444,13 +497,13 @@ def Unquote(quoted_string):
         return quoted_string
     assert quoted_string[0] == quoted_string[-1]
     return_list = []
-    i = 1 # skip initial char
-    while i < len(quoted_string)-1:
+    i = 1  # skip initial char
+    while i < len(quoted_string) - 1:
         char = quoted_string[i]
         if char == "\\":
             # quoted section
-            assert quoted_string[i+1] == "x"
-            return_list.append(chr(int(quoted_string[i+2:i+4], 16)))
+            assert quoted_string[i + 1] == "x"
+            return_list.append(chr(int(quoted_string[i + 2:i + 4], 16)))
             i += 4
         else:
             return_list.append(char)
