@@ -38,6 +38,10 @@ from duplicity import globals
 # instance
 _defaultLock = threading.Lock()
 _defaultInstance = None
+# backup the initial tmp dir path because we will force tempfile
+# later to use our generated _defaultInstance.dir() as temproot
+_initialSystemTempRoot = tempfile.gettempdir()
+
 
 def default():
     """
@@ -54,12 +58,15 @@ def default():
 
     _defaultLock.acquire()
     try:
-        if _defaultInstance is None:
-            _defaultInstance = TemporaryDirectory(temproot = globals.temproot)
+        if _defaultInstance is None or _defaultInstance.dir() is None:
+            _defaultInstance = TemporaryDirectory(temproot=globals.temproot)
+            # set the temp dir to be the default in tempfile module from now on
+            tempfile.tempdir = _defaultInstance.dir()
         return _defaultInstance
     finally:
         _defaultLock.release()
 
+
 class TemporaryDirectory:
     """
     A temporary directory.
@@ -106,7 +113,7 @@ class TemporaryDirectory:
     well. The impact should however be limited to the removal of an
     'attackers' file.
     """
-    def __init__(self, temproot = None):
+    def __init__(self, temproot=None):
         """
         Create a new TemporaryDirectory backed by a unique and
         securely created file system directory.
@@ -114,9 +121,15 @@ class TemporaryDirectory:
         tempbase - The temp root directory, or None to use system
         default (recommended).
         """
+        if temproot is None:
+            if globals.temproot:
+                temproot = globals.temproot
+            else:
+                global _initialSystemTempRoot
+                temproot = _initialSystemTempRoot
         self.__dir = tempfile.mkdtemp("-tempdir", "duplicity-", temproot)
 
-        log.Info(_("Using temporary directory %s") % (self.__dir,))
+        log.Info(_("Using temporary directory %s") % util.ufn(self.__dir))
 
         # number of mktemp()/mkstemp() calls served so far
         self.__tempcount = 0
@@ -160,7 +173,7 @@ class TemporaryDirectory:
             suffix = "-%d" % (self.__tempcount,)
             filename = tempfile.mktemp(suffix, "mktemp-", self.__dir)
 
-            log.Debug(_("Registering (mktemp) temporary file %s") % (filename,))
+            log.Debug(_("Registering (mktemp) temporary file %s") % util.ufn(filename))
             self.__pending[filename] = None
         finally:
             self.__lock.release()
@@ -182,7 +195,7 @@ class TemporaryDirectory:
             suffix = "-%d" % (self.__tempcount,)
             fd, filename = tempfile.mkstemp(suffix, "mkstemp-", self.__dir)
 
-            log.Debug(_("Registering (mkstemp) temporary file %s") % (filename,))
+            log.Debug(_("Registering (mkstemp) temporary file %s") % util.ufn(filename))
             self.__pending[filename] = None
         finally:
             self.__lock.release()
@@ -213,11 +226,11 @@ class TemporaryDirectory:
         """
         self.__lock.acquire()
         try:
-            if self.__pending.has_key(fname):
-                log.Debug(_("Forgetting temporary file %s") % (fname, ))
+            if fname in self.__pending:
+                log.Debug(_("Forgetting temporary file %s") % util.ufn(fname))
                 del(self.__pending[fname])
             else:
-                log.Warn(_("Attempt to forget unknown tempfile %s - this is probably a bug.") % (fname,))
+                log.Warn(_("Attempt to forget unknown tempfile %s - this is probably a bug.") % util.ufn(fname))
                 pass
         finally:
             self.__lock.release()
@@ -233,18 +246,19 @@ class TemporaryDirectory:
         """
         self.__lock.acquire()
         try:
-            if not self.__dir is None:
+            if self.__dir is not None:
                 for file in self.__pending.keys():
                     try:
-                        log.Debug(_("Removing still remembered temporary file %s") % (file,))
+                        log.Debug(_("Removing still remembered temporary file %s") % util.ufn(file))
                         util.ignore_missing(os.unlink, file)
                     except Exception:
-                        log.Info(_("Cleanup of temporary file %s failed") % (file,))
+                        log.Info(_("Cleanup of temporary file %s failed") % util.ufn(file))
                         pass
                 try:
                     os.rmdir(self.__dir)
                 except Exception:
-                    log.Warn(_("Cleanup of temporary directory %s failed - this is probably a bug.") % (self.__dir,))
+                    log.Warn(_("Cleanup of temporary directory %s failed - "
+                               "this is probably a bug.") % util.ufn(self.__dir))
                     pass
                 self.__pending = None
                 self.__dir = None
